=== modified file 'NEWS'
--- NEWS	2006-04-28 09:35:20 +0000
+++ NEWS	2006-05-03 08:05:05 +0000
@@ -69,7 +69,9 @@
     * Improve output of 'info' command. Show all relevant locations related to
       working tree, branch and repository. Use kibibytes for binary quantities.
       Fix off-by-one error in missing revisions of working tree.  Make 'info'
-      work on branches, repositories and remote locations.  (Olaf Conradi)
+      work on branches, repositories and remote locations.  Show locations
+      relative to the shared repository, if applicable.  Show locking status
+      of locations.  (Olaf Conradi)
 
     * Diff and merge now safely handle binary files. (Aaron Bentley)
 
@@ -166,6 +168,9 @@
       useful for testing or for overriding the bzrlib.progress heuristic.
       (Robert Collins)
 
+    * New API method get_physical_lock_status() to query locks present on a
+      transport.  (Olaf Conradi)
+
   TESTING:
 
     * SFTP tests now shortcut the SSH negotiation, reducing test overhead

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2006-04-13 22:54:24 +0000
+++ bzrlib/branch.py	2006-04-29 23:10:05 +0000
@@ -154,10 +154,13 @@
         assert cfg.get_option("nickname") == nick
 
     nick = property(_get_nick, _set_nick)
-        
+
+    def is_locked(self):
+        raise NotImplementedError('is_locked is abstract')
+
     def lock_write(self):
         raise NotImplementedError('lock_write is abstract')
-        
+
     def lock_read(self):
         raise NotImplementedError('lock_read is abstract')
 
@@ -168,6 +171,9 @@
         """Return lock mode for the Branch: 'r', 'w' or None"""
         raise NotImplementedError(self.peek_lock_mode)
 
+    def get_physical_lock_status(self):
+        raise NotImplementedError('get_physical_lock_status is abstract')
+
     def abspath(self, name):
         """Return absolute filename for something in the branch
         
@@ -939,6 +945,9 @@
         tree = self.repository.revision_tree(self.last_revision())
         return tree.inventory.root.file_id
 
+    def is_locked(self):
+        return self.control_files.is_locked()
+
     def lock_write(self):
         # TODO: test for failed two phase locks. This is known broken.
         self.control_files.lock_write()
@@ -960,6 +969,9 @@
         else:
             return self.control_files._lock_mode
 
+    def get_physical_lock_status(self):
+        return self.control_files.get_physical_lock_status()
+
     @needs_read_lock
     def print_file(self, file, revision_id):
         """See Branch.print_file."""

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-04-29 09:19:50 +0000
+++ bzrlib/builtins.py	2006-05-03 08:05:05 +0000
@@ -766,15 +766,22 @@
 
 
 class cmd_info(Command):
-    """Show statistical information about a branch."""
-    takes_args = ['branch?']
+    """Show information about a working tree, branch or repository.
+
+    This command will show all known locations and formats associated to the
+    tree, branch or repository.  Statistical information is included with
+    each report.
+
+    Branches and working trees will also report any missing revisions.
+    """
+    takes_args = ['location?']
     takes_options = ['verbose']
-    
+
     @display_command
-    def run(self, branch=None, verbose=False):
-        import bzrlib.info
-        bzrlib.info.show_bzrdir_info(bzrdir.BzrDir.open_containing(branch)[0],
-                                     verbose=verbose)
+    def run(self, location=None, verbose=False):
+        from bzrlib.info import show_bzrdir_info
+        show_bzrdir_info(bzrdir.BzrDir.open_containing(location)[0],
+                         verbose=verbose)
 
 
 class cmd_remove(Command):

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2006-04-11 01:28:20 +0000
+++ bzrlib/info.py	2006-05-01 12:32:26 +0000
@@ -25,18 +25,10 @@
 from bzrlib.errors import (NoWorkingTree, NotBranchError,
                            NoRepositoryPresent, NotLocalUrl)
 from bzrlib.missing import find_unmerged
-from bzrlib.osutils import format_date
+import bzrlib.osutils as osutils
 from bzrlib.symbol_versioning import *
 
 
-def _countiter(it):
-    # surely there's a builtin for this?
-    i = 0
-    for j in it:
-        i += 1
-    return i        
-
-
 def plural(n, base='', pl=None):
     if n == 1:
         return base
@@ -46,35 +38,77 @@
         return 's'
 
 
-def _show_location_info(repository=None, branch=None, working=None):
+def _repo_relpath(repo_path, path):
+    """Return path with common prefix of repository path removed.
+
+    If path is not part of the repository, the original path is returned.
+    If path is equal to the repository, the current directory marker '.' is
+    returned.
+    """
+    path = osutils.normalizepath(path)
+    repo_path = osutils.normalizepath(repo_path)
+    if path == repo_path:
+        return '.'
+    if osutils.is_inside(repo_path, path):
+        return osutils.relpath(repo_path, path)
+    return path
+
+
+def _show_location_info(repository, branch=None, working=None):
     """Show known locations for working, branch and repository."""
+    repository_path = repository.bzrdir.root_transport.base
     print 'Location:'
-    if working and branch and working.bzrdir != branch.bzrdir:
-        # Lightweight checkout
-        print '       checkout root: %s' % (
-            working.bzrdir.root_transport.base)
-        print '  checkout of branch: %s' % (
-            branch.bzrdir.root_transport.base)
+    if working and branch:
+        working_path = working.bzrdir.root_transport.base
+        branch_path = branch.bzrdir.root_transport.base
+        if working_path != branch_path:
+            # lightweight checkout
+            print '  light checkout root: %s' % working_path
+            if repository.is_shared():
+                # lightweight checkout of branch in shared repository
+                print '    shared repository: %s' % repository_path
+                print '    repository branch: %s' % (
+                    _repo_relpath(repository_path, branch_path))
+            else:
+                # lightweight checkout of standalone branch
+                print '   checkout of branch: %s' % branch_path
+        elif repository.is_shared():
+            # branch with tree inside shared repository
+            print '    shared repository: %s' % repository_path
+            print '  repository checkout: %s' % (
+                _repo_relpath(repository_path, branch_path))
+        elif branch.get_bound_location():
+            # normal checkout
+            print '       checkout root: %s' % working_path
+            print '  checkout of branch: %s' % branch.get_bound_location()
+        else:
+            # standalone
+            print '  branch root: %s' % working_path
     elif branch:
-        # Standalone or bound branch (normal checkout)
-        print '         branch root: %s' % (
-            branch.bzrdir.root_transport.base)
-        if branch.get_bound_location():
-            print '     bound to branch: %s' % branch.get_bound_location()
-
-    if repository and (not branch or repository.bzrdir != branch.bzrdir):
-        if repository.is_shared():
-            print '   shared repository: %s' % (
-                repository.bzrdir.root_transport.base)
-        else:
-            print '          repository: %s' % (
-                repository.bzrdir.root_transport.base)
-
-    if branch:
+        # branch is part of shared repository
+        assert repository.is_shared()
+        branch_path = branch.bzrdir.root_transport.base
+        print '  shared repository: %s' % repository_path
+        print '  repository branch: %s' % (
+            _repo_relpath(repository_path, branch_path))
+    else:
+        # shared repository
+        assert repository.is_shared()
+        print '  shared repository: %s' % repository_path
+
+
+def _show_related_info(branch):
+    """Show parent and push location of branch."""
+    if branch.get_parent() or branch.get_push_location():
+        print
+        print 'Related branches:'
         if branch.get_parent():
-            print '       parent branch: %s' % branch.get_parent()
+            if branch.get_push_location():
+                print '      parent branch: %s' % branch.get_parent()
+            else:
+                print '  parent branch: %s' % branch.get_parent()
         if branch.get_push_location():
-            print '      push to branch: %s' % branch.get_push_location()
+            print '  publish to branch: %s' % branch.get_push_location()
 
 
 def _show_format_info(control=None, repository=None, branch=None, working=None):
@@ -91,6 +125,33 @@
         print '    repository: %s' % repository._format.get_format_description()
 
 
+def _show_locking_info(repository, branch=None, working=None):
+    """Show locking status of working, branch and repository."""
+    if (repository.get_physical_lock_status() or
+        (branch and branch.get_physical_lock_status()) or
+        (working and working.get_physical_lock_status())):
+        print
+        print 'Lock status:'
+        if working:
+            if working.get_physical_lock_status():
+                status = 'locked'
+            else:
+                status = 'unlocked'
+            print '  working tree: %s' % status
+        if branch:
+            if branch.get_physical_lock_status():
+                status = 'locked'
+            else:
+                status = 'unlocked'
+            print '        branch: %s' % status
+        if repository:
+            if repository.get_physical_lock_status():
+                status = 'locked'
+            else:
+                status = 'unlocked'
+            print '    repository: %s' % status
+
+
 def _show_missing_revisions_branch(branch):
     """Show missing master revisions in branch."""
     # Try with inaccessible branch ?
@@ -170,12 +231,12 @@
         firstrev = repository.get_revision(history[0])
         age = int((time.time() - firstrev.timestamp) / 3600 / 24)
         print '  %8d day%s old' % (age, plural(age))
-        print '   first revision: %s' % format_date(firstrev.timestamp,
-                                                    firstrev.timezone)
+        print '   first revision: %s' % osutils.format_date(firstrev.timestamp,
+                                                            firstrev.timezone)
 
         lastrev = repository.get_revision(history[-1])
-        print '  latest revision: %s' % format_date(lastrev.timestamp,
-                                                    lastrev.timezone)
+        print '  latest revision: %s' % osutils.format_date(lastrev.timestamp,
+                                                            lastrev.timezone)
 
 #     print
 #     print 'Text store:'
@@ -248,7 +309,7 @@
     except NoRepositoryPresent:
         pass
 
-    # Return silently, cmd_info returns NotBranchError if no bzrdir
+    # Return silently, cmd_info already returned NotBranchError if no bzrdir
     # could be opened.
 
 
@@ -259,7 +320,9 @@
     control = working.bzrdir
 
     _show_location_info(repository, branch, working)
+    _show_related_info(branch)
     _show_format_info(control, repository, branch, working)
+    _show_locking_info(repository, branch, working)
     _show_missing_revisions_branch(branch)
     _show_missing_revisions_working(working)
     _show_working_stats(working)
@@ -273,17 +336,20 @@
     control = branch.bzrdir
 
     _show_location_info(repository, branch)
+    _show_related_info(branch)
     _show_format_info(control, repository, branch)
+    _show_locking_info(repository, branch)
     _show_missing_revisions_branch(branch)
     _show_branch_stats(branch, verbose)
     _show_repository_stats(repository)
 
 
 def show_repository_info(repository, verbose):
-    """Output to stdout the 'info' for branch."""
+    """Output to stdout the 'info' for repository."""
     control = repository.bzrdir
 
     _show_location_info(repository)
     _show_format_info(control, repository)
+    _show_locking_info(repository)
     _show_repository_info(repository)
     _show_repository_stats(repository)

=== modified file 'bzrlib/lockable_files.py'
--- bzrlib/lockable_files.py	2006-04-18 22:41:16 +0000
+++ bzrlib/lockable_files.py	2006-04-30 20:41:47 +0000
@@ -21,7 +21,7 @@
 import bzrlib
 from bzrlib.decorators import *
 import bzrlib.errors as errors
-from bzrlib.errors import LockError, ReadOnlyError
+from bzrlib.errors import BzrError
 from bzrlib.osutils import file_iterator, safe_unicode
 from bzrlib.symbol_versioning import *
 from bzrlib.trace import mutter, note
@@ -208,7 +208,7 @@
         # and potentially a remote locking protocol
         if self._lock_mode:
             if self._lock_mode != 'w' or not self.get_transaction().writeable():
-                raise ReadOnlyError(self)
+                raise errors.ReadOnlyError(self)
             self._lock_count += 1
         else:
             self._lock.lock_write()
@@ -251,6 +251,18 @@
         """Return true if this LockableFiles group is locked"""
         return self._lock_count >= 1
 
+    def get_physical_lock_status(self):
+        """Return physical lock status.
+        
+        Returns true if a lock is held on the transport. If no lock is held, or
+        the underlying locking mechanism does not support querying lock
+        status, false is returned.
+        """
+        try:
+            return self._lock.peek() is not None
+        except NotImplementedError:
+            return False
+
     def get_transaction(self):
         """Return the current active transaction.
 
@@ -306,6 +318,9 @@
         self._lock.unlock()
         self._lock = None
 
+    def peek(self):
+        raise NotImplementedError()
+
     def create(self, mode=None):
         """Create lock mechanism"""
         # for old-style locks, create the file now

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-04-19 23:32:08 +0000
+++ bzrlib/repository.py	2006-04-29 23:10:05 +0000
@@ -159,14 +159,17 @@
         # TODO: make sure to construct the right store classes, etc, depending
         # on whether escaping is required.
 
+    def is_locked(self):
+        return self.control_files.is_locked()
+
     def lock_write(self):
         self.control_files.lock_write()
 
     def lock_read(self):
         self.control_files.lock_read()
 
-    def is_locked(self):
-        return self.control_files.is_locked()
+    def get_physical_lock_status(self):
+        return self.control_files.get_physical_lock_status()
 
     @needs_read_lock
     def missing_revision_ids(self, other, revision_id=None):

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2006-04-19 23:32:08 +0000
+++ bzrlib/tests/blackbox/test_info.py	2006-05-01 13:15:09 +0000
@@ -29,6 +29,11 @@
 
 class TestInfo(ExternalBase):
 
+    def test_info_non_existing(self):
+        out, err = self.runbzr('info /i/do/not/exist/', retcode=3)
+        self.assertEqual(out, '')
+        self.assertEqual(err, 'bzr: ERROR: Not a branch: /i/do/not/exist/\n')
+
     def test_info_standalone(self):
         transport = self.get_transport()
 
@@ -43,7 +48,7 @@
         out, err = self.runbzr('info standalone')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
+  branch root: %s
 
 Format:
        control: All-in-one format 6
@@ -79,9 +84,11 @@
         out, err = self.runbzr('info branch --verbose')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-       parent branch: %s
-      push to branch: %s
+  branch root: %s
+
+Related branches:
+      parent branch: %s
+  publish to branch: %s
 
 Format:
        control: All-in-one format 6
@@ -108,27 +115,32 @@
 
 Revision store:
          1 revision
-         0 KiB
-""" % (branch2.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
-       branch1.bzrdir.root_transport.base,
-       datestring_first, datestring_first), out)
+         %d KiB
+""" % (branch2.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first,
+       # poking at _revision_store isn't all that clean, but neither is
+       # having the ui test dependent on the exact overhead of a given store.
+       branch2.repository._revision_store.total_size(
+        branch2.repository.get_transaction())[1] / 1024,
+       ), out)
         self.assertEqual('', err)
 
         # Branch and bind to standalone, needs upgrade to metadir
         # (creates backup as unknown)
-        # XXX: I can't get this to work through API
-        self.runbzr('branch standalone bound')
-        #branch3 = branch1.bzrdir.sprout('bound').open_branch()
-        self.runbzr('upgrade --format=metadir bound')
-        #bzrlib.upgrade.upgrade('bound', 'metadir')
+        branch3 = branch1.bzrdir.sprout('bound').open_branch()
+        bzrlib.upgrade.upgrade('bound', bzrlib.bzrdir.BzrDirMetaFormat1())
         branch3 = bzrlib.bzrdir.BzrDir.open('bound').open_branch()
         branch3.bind(branch1)
         out, err = self.runbzr('info bound')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-     bound to branch: %s
-       parent branch: %s
+       checkout root: %s
+  checkout of branch: %s
+
+Related branches:
+  parent branch: %s
 
 Format:
        control: Meta directory format 1
@@ -155,7 +167,8 @@
 Revision store:
          1 revision
          %d KiB
-""" % (branch3.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+""" % (branch3.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
        branch3.repository._format.get_format_description(),
        datestring_first, datestring_first,
@@ -176,8 +189,8 @@
         out, err = self.runbzr('info checkout --verbose')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-     bound to branch: %s
+       checkout root: %s
+  checkout of branch: %s
 
 Format:
        control: Meta directory format 1
@@ -205,7 +218,8 @@
 Revision store:
          1 revision
          %d KiB
-""" % (branch4.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+""" % (branch4.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
        branch4.repository._format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
@@ -228,8 +242,8 @@
         out, err = self.runbzr('info lightcheckout')
         self.assertEqualDiff(
 """Location:
-       checkout root: %s
-  checkout of branch: %s
+  light checkout root: %s
+   checkout of branch: %s
 
 Format:
        control: Meta directory format 1
@@ -256,8 +270,10 @@
 Revision store:
          1 revision
          0 KiB
-""" % (tree5.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
-       datestring_first, datestring_first), out)
+""" % (tree5.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first,
+       ), out)
         self.assertEqual('', err)
 
         # Update initial standalone branch
@@ -271,9 +287,11 @@
         out, err = self.runbzr('info branch')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-       parent branch: %s
-      push to branch: %s
+  branch root: %s
+
+Related branches:
+      parent branch: %s
+  publish to branch: %s
 
 Format:
        control: All-in-one format 6
@@ -300,18 +318,22 @@
 Revision store:
          1 revision
          0 KiB
-""" % (branch2.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
-       branch1.bzrdir.root_transport.base,
-       datestring_first, datestring_first), out)
+""" % (branch2.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_first,
+       ), out)
         self.assertEqual('', err)
 
         # Out of date bound branch
         out, err = self.runbzr('info bound')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-     bound to branch: %s
-       parent branch: %s
+       checkout root: %s
+  checkout of branch: %s
+
+Related branches:
+  parent branch: %s
 
 Format:
        control: Meta directory format 1
@@ -340,7 +362,8 @@
 Revision store:
          1 revision
          %d KiB
-""" % (branch3.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+""" % (branch3.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
        branch3.repository._format.get_format_description(),
        datestring_first, datestring_first,
@@ -355,8 +378,8 @@
         out, err = self.runbzr('info checkout')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-     bound to branch: %s
+       checkout root: %s
+  checkout of branch: %s
 
 Format:
        control: Meta directory format 1
@@ -385,7 +408,8 @@
 Revision store:
          1 revision
          %d KiB
-""" % (branch4.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
+""" % (branch4.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
        branch4.repository._format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
@@ -399,8 +423,8 @@
         out, err = self.runbzr('info lightcheckout --verbose')
         self.assertEqualDiff(
 """Location:
-       checkout root: %s
-  checkout of branch: %s
+  light checkout root: %s
+   checkout of branch: %s
 
 Format:
        control: Meta directory format 1
@@ -430,8 +454,10 @@
 Revision store:
          2 revisions
          0 KiB
-""" % (tree5.bzrdir.root_transport.base, branch1.bzrdir.root_transport.base,
-       datestring_first, datestring_last), out)
+""" % (tree5.bzrdir.root_transport.base,
+       branch1.bzrdir.root_transport.base,
+       datestring_first, datestring_last,
+       ), out)
         self.assertEqual('', err)
 
     def test_info_shared_repository(self):
@@ -445,7 +471,7 @@
         out, err = self.runbzr('info repo')
         self.assertEqualDiff(
 """Location:
-   shared repository: %s
+  shared repository: %s
 
 Format:
        control: Meta directory format 1
@@ -454,8 +480,9 @@
 Revision store:
          0 revisions
          0 KiB
-""" % (repo.bzrdir.root_transport.base, repo._format.get_format_description()),
-            out)
+""" % (repo.bzrdir.root_transport.base,
+       repo._format.get_format_description(),
+       ), out)
         self.assertEqual('', err)
 
         # Create branch inside shared repository
@@ -464,8 +491,8 @@
         out, err = self.runbzr('info repo/branch')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-   shared repository: %s
+  shared repository: %s
+  repository branch: branch
 
 Format:
        control: Meta directory format 1
@@ -478,8 +505,7 @@
 Revision store:
          0 revisions
          0 KiB
-""" % (branch1.bzrdir.root_transport.base,
-       repo.bzrdir.root_transport.base,
+""" % (repo.bzrdir.root_transport.base,
        repo._format.get_format_description(),
        ), out)
         self.assertEqual('', err)
@@ -495,9 +521,9 @@
         out, err = self.runbzr('info tree/lightcheckout')
         self.assertEqualDiff(
 """Location:
-       checkout root: %s
-  checkout of branch: %s
-   shared repository: %s
+  light checkout root: %s
+    shared repository: %s
+    repository branch: branch
 
 Format:
        control: Meta directory format 1
@@ -522,7 +548,6 @@
          0 revisions
          0 KiB
 """ % (tree2.bzrdir.root_transport.base,
-       branch1.bzrdir.root_transport.base,
        repo.bzrdir.root_transport.base,
        repo._format.get_format_description(),
        ), out)
@@ -536,8 +561,8 @@
         out, err = self.runbzr('info tree/checkout --verbose')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-     bound to branch: %s
+       checkout root: %s
+  checkout of branch: %s
 
 Format:
        control: Meta directory format 1
@@ -577,9 +602,9 @@
         out, err = self.runbzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
 """Location:
-       checkout root: %s
-  checkout of branch: %s
-   shared repository: %s
+  light checkout root: %s
+    shared repository: %s
+    repository branch: branch
 
 Format:
        control: Meta directory format 1
@@ -608,7 +633,6 @@
          1 revision
          %d KiB
 """ % (tree2.bzrdir.root_transport.base,
-       branch1.bzrdir.root_transport.base,
        repo.bzrdir.root_transport.base,
        repo._format.get_format_description(),
        datestring_first, datestring_first,
@@ -622,8 +646,8 @@
         out, err = self.runbzr('info tree/checkout')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-     bound to branch: %s
+       checkout root: %s
+  checkout of branch: %s
 
 Format:
        control: Meta directory format 1
@@ -662,8 +686,8 @@
         out, err = self.runbzr('info tree/checkout --verbose')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-     bound to branch: %s
+       checkout root: %s
+  checkout of branch: %s
 
 Format:
        control: Meta directory format 1
@@ -707,9 +731,9 @@
         out, err = self.runbzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
 """Location:
-       checkout root: %s
-  checkout of branch: %s
-   shared repository: %s
+  light checkout root: %s
+    shared repository: %s
+    repository branch: branch
 
 Format:
        control: Meta directory format 1
@@ -740,7 +764,6 @@
          2 revisions
          %d KiB
 """ % (tree2.bzrdir.root_transport.base,
-       branch1.bzrdir.root_transport.base,
        repo.bzrdir.root_transport.base,
        repo._format.get_format_description(),
        datestring_first, datestring_last,
@@ -754,8 +777,8 @@
         out, err = self.runbzr('info repo/branch --verbose')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-   shared repository: %s
+  shared repository: %s
+  repository branch: branch
 
 Format:
        control: Meta directory format 1
@@ -772,8 +795,7 @@
 Revision store:
          2 revisions
          %d KiB
-""" % (branch1.bzrdir.root_transport.base,
-       repo.bzrdir.root_transport.base,
+""" % (repo.bzrdir.root_transport.base,
        repo._format.get_format_description(),
        datestring_first, datestring_last,
        # poking at _revision_store isn't all that clean, but neither is
@@ -786,7 +808,7 @@
         out, err = self.runbzr('info repo')
         self.assertEqualDiff(
 """Location:
-   shared repository: %s
+  shared repository: %s
 
 Format:
        control: Meta directory format 1
@@ -800,8 +822,7 @@
        # poking at _revision_store isn't all that clean, but neither is
        # having the ui test dependent on the exact overhead of a given store.
        repo._revision_store.total_size(repo.get_transaction())[1] / 1024,
-       ),
-       out)
+       ), out)
         self.assertEqual('', err)
 
         bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
@@ -817,7 +838,7 @@
         out, err = self.runbzr('info repo')
         self.assertEqualDiff(
 """Location:
-   shared repository: %s
+  shared repository: %s
 
 Format:
        control: Meta directory format 1
@@ -842,8 +863,8 @@
         out, err = self.runbzr('info repo/branch1 --verbose')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-   shared repository: %s
+    shared repository: %s
+  repository checkout: branch1
 
 Format:
        control: Meta directory format 1
@@ -868,8 +889,7 @@
 Revision store:
          0 revisions
          0 KiB
-""" % (branch1.bzrdir.root_transport.base,
-       repo.bzrdir.root_transport.base,
+""" % (repo.bzrdir.root_transport.base,
        repo._format.get_format_description(),
        ), out)
         self.assertEqual('', err)
@@ -884,8 +904,8 @@
         out, err = self.runbzr('info repo/branch1')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-   shared repository: %s
+    shared repository: %s
+  repository checkout: branch1
 
 Format:
        control: Meta directory format 1
@@ -912,7 +932,7 @@
 Revision store:
          1 revision
          %d KiB
-""" % (branch1.bzrdir.root_transport.base, repo.bzrdir.root_transport.base,
+""" % (repo.bzrdir.root_transport.base,
        repo._format.get_format_description(),
        datestring_first, datestring_first,
        # poking at _revision_store isn't all that clean, but neither is
@@ -925,9 +945,11 @@
         out, err = self.runbzr('info repo/branch2 --verbose')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-   shared repository: %s
-       parent branch: %s
+    shared repository: %s
+  repository checkout: branch2
+
+Related branches:
+  parent branch: %s
 
 Format:
        control: Meta directory format 1
@@ -952,7 +974,7 @@
 Revision store:
          1 revision
          %d KiB
-""" % (branch2.bzrdir.root_transport.base, repo.bzrdir.root_transport.base,
+""" % (repo.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
        repo._format.get_format_description(),
        # poking at _revision_store isn't all that clean, but neither is
@@ -967,9 +989,11 @@
         out, err = self.runbzr('info repo/branch2')
         self.assertEqualDiff(
 """Location:
-         branch root: %s
-   shared repository: %s
-       parent branch: %s
+    shared repository: %s
+  repository checkout: branch2
+
+Related branches:
+  parent branch: %s
 
 Format:
        control: Meta directory format 1
@@ -996,8 +1020,7 @@
 Revision store:
          1 revision
          %d KiB
-""" % (branch2.bzrdir.root_transport.base,
-       repo.bzrdir.root_transport.base,
+""" % (repo.bzrdir.root_transport.base,
        branch1.bzrdir.root_transport.base,
        repo._format.get_format_description(),
        datestring_first, datestring_first,
@@ -1011,7 +1034,7 @@
         out, err = self.runbzr('info repo')
         self.assertEqualDiff(
 """Location:
-   shared repository: %s
+  shared repository: %s
 
 Format:
        control: Meta directory format 1
@@ -1032,3 +1055,503 @@
         self.assertEqual('', err)
 
         bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+    
+    def test_info_shared_repository_with_tree_in_root(self):
+        old_format = bzrlib.bzrdir.BzrDirFormat.get_default_format()
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(bzrlib.bzrdir.BzrDirMetaFormat1())
+        transport = self.get_transport()
+
+        # Create shared repository with working trees
+        repo = self.make_repository('repo', shared=True)
+        repo.set_make_working_trees(True)
+        out, err = self.runbzr('info repo')
+        self.assertEqualDiff(
+"""Location:
+  shared repository: %s
+
+Format:
+       control: Meta directory format 1
+    repository: %s
+
+Create working tree for new branches inside the repository.
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (repo.bzrdir.root_transport.base,
+       repo._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+
+        # Create branch in root of repository
+        control = repo.bzrdir
+        branch = control.create_branch()
+        control.create_workingtree()
+        out, err = self.runbzr('info repo')
+        self.assertEqualDiff(
+"""Location:
+    shared repository: %s
+  repository checkout: .
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: %s
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (repo.bzrdir.root_transport.base,
+       repo._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+
+        bzrlib.bzrdir.BzrDirFormat.set_default_format(old_format)
+
+    def test_info_locking(self):
+        transport = self.get_transport()
+        # Create shared repository with a branch
+        repo = self.make_repository('repo', shared=True,
+                                    format=bzrlib.bzrdir.BzrDirMetaFormat1())
+        repo.set_make_working_trees(False)
+        repo.bzrdir.root_transport.mkdir('branch')
+        repo_branch = repo.bzrdir.create_branch_convenience('repo/branch',
+                                    format=bzrlib.bzrdir.BzrDirMetaFormat1())
+        # Do a heavy checkout
+        transport.mkdir('tree')
+        transport.mkdir('tree/checkout')
+        co_branch = bzrlib.bzrdir.BzrDir.create_branch_convenience('tree/checkout',
+                                    format=bzrlib.bzrdir.BzrDirMetaFormat1())
+        co_branch.bind(repo_branch)
+        # Do a light checkout of the heavy one
+        transport.mkdir('tree/lightcheckout')
+        lco_dir = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('tree/lightcheckout')
+        bzrlib.branch.BranchReferenceFormat().initialize(lco_dir, co_branch)
+        lco_dir.create_workingtree()
+        lco_tree = lco_dir.open_workingtree()
+
+        # Test all permutations of locking the working tree, branch and repository
+        # W B R
+
+        # U U U
+        out, err = self.runbzr('info tree/lightcheckout')
+        self.assertEqualDiff(
+"""Location:
+  light checkout root: %s
+   checkout of branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: %s
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (lco_tree.bzrdir.root_transport.base,
+       lco_tree.branch.bzrdir.root_transport.base,
+       lco_tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        # U U L
+        lco_tree.branch.repository.lock_write()
+        out, err = self.runbzr('info tree/lightcheckout')
+        self.assertEqualDiff(
+"""Location:
+  light checkout root: %s
+   checkout of branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: %s
+
+Lock status:
+  working tree: unlocked
+        branch: unlocked
+    repository: locked
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (lco_tree.bzrdir.root_transport.base,
+       lco_tree.branch.bzrdir.root_transport.base,
+       lco_tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        lco_tree.branch.repository.unlock()
+        # U L L
+        lco_tree.branch.lock_write()
+        out, err = self.runbzr('info tree/lightcheckout')
+        self.assertEqualDiff(
+"""Location:
+  light checkout root: %s
+   checkout of branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: %s
+
+Lock status:
+  working tree: unlocked
+        branch: locked
+    repository: locked
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (lco_tree.bzrdir.root_transport.base,
+       lco_tree.branch.bzrdir.root_transport.base,
+       lco_tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        lco_tree.branch.unlock()
+        # L L L
+        lco_tree.lock_write()
+        out, err = self.runbzr('info tree/lightcheckout')
+        self.assertEqualDiff(
+"""Location:
+  light checkout root: %s
+   checkout of branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: %s
+
+Lock status:
+  working tree: locked
+        branch: locked
+    repository: locked
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (lco_tree.bzrdir.root_transport.base,
+       lco_tree.branch.bzrdir.root_transport.base,
+       lco_tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        lco_tree.unlock()
+        # L L U
+        lco_tree.lock_write()
+        lco_tree.branch.repository.unlock()
+        out, err = self.runbzr('info tree/lightcheckout')
+        self.assertEqualDiff(
+"""Location:
+  light checkout root: %s
+   checkout of branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: %s
+
+Lock status:
+  working tree: locked
+        branch: locked
+    repository: unlocked
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (lco_tree.bzrdir.root_transport.base,
+       lco_tree.branch.bzrdir.root_transport.base,
+       lco_tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        lco_tree.branch.repository.lock_write()
+        lco_tree.unlock()
+        # L U U
+        lco_tree.lock_write()
+        lco_tree.branch.unlock()
+        out, err = self.runbzr('info tree/lightcheckout')
+        self.assertEqualDiff(
+"""Location:
+  light checkout root: %s
+   checkout of branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: %s
+
+Lock status:
+  working tree: locked
+        branch: unlocked
+    repository: unlocked
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (lco_tree.bzrdir.root_transport.base,
+       lco_tree.branch.bzrdir.root_transport.base,
+       lco_tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        lco_tree.branch.lock_write()
+        lco_tree.unlock()
+        # L U L
+        lco_tree.lock_write()
+        lco_tree.branch.unlock()
+        lco_tree.branch.repository.lock_write()
+        out, err = self.runbzr('info tree/lightcheckout')
+        self.assertEqualDiff(
+"""Location:
+  light checkout root: %s
+   checkout of branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: %s
+
+Lock status:
+  working tree: locked
+        branch: unlocked
+    repository: locked
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (lco_tree.bzrdir.root_transport.base,
+       lco_tree.branch.bzrdir.root_transport.base,
+       lco_tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        lco_tree.branch.repository.unlock()
+        lco_tree.branch.lock_write()
+        lco_tree.unlock()
+        # U L U
+        lco_tree.branch.lock_write()
+        lco_tree.branch.repository.unlock()
+        out, err = self.runbzr('info tree/lightcheckout')
+        self.assertEqualDiff(
+"""Location:
+  light checkout root: %s
+   checkout of branch: %s
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 3
+        branch: Branch format 5
+    repository: %s
+
+Lock status:
+  working tree: unlocked
+        branch: locked
+    repository: unlocked
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (lco_tree.bzrdir.root_transport.base,
+       lco_tree.branch.bzrdir.root_transport.base,
+       lco_tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        lco_tree.branch.repository.lock_write()
+        lco_tree.branch.unlock()
+
+    def test_info_locking_oslocks(self):
+        tree = self.make_branch_and_tree('branch',
+                                         format=bzrlib.bzrdir.BzrDirFormat6())
+
+        # Test all permutations of locking the working tree, branch and repository
+        # XXX: Well not yet, as we can't query oslocks yet. Currently, it's
+        # implemented by raising NotImplementedError and get_physical_lock_status()
+        # always returns false. This makes bzr info hide the lock status.  (Olaf)
+        # W B R
+
+        # U U U
+        out, err = self.runbzr('info branch')
+        self.assertEqualDiff(
+"""Location:
+  branch root: %s
+
+Format:
+       control: All-in-one format 6
+  working tree: Working tree format 2
+        branch: Branch format 4
+    repository: %s
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (tree.bzrdir.root_transport.base,
+       tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        # L L L
+        tree.lock_write()
+        out, err = self.runbzr('info branch')
+        self.assertEqualDiff(
+"""Location:
+  branch root: %s
+
+Format:
+       control: All-in-one format 6
+  working tree: Working tree format 2
+        branch: Branch format 4
+    repository: %s
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+
+Branch history:
+         0 revisions
+
+Revision store:
+         0 revisions
+         0 KiB
+""" % (tree.bzrdir.root_transport.base,
+       tree.branch.repository._format.get_format_description(),
+       ), out)
+        self.assertEqual('', err)
+        tree.unlock()

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-04-21 01:41:02 +0000
+++ bzrlib/workingtree.py	2006-05-03 08:05:05 +0000
@@ -1031,6 +1031,9 @@
         """
         return self.branch.last_revision()
 
+    def is_locked(self):
+        return self._control_files.is_locked()
+
     def lock_read(self):
         """See Branch.lock_read, and WorkingTree.unlock."""
         self.branch.lock_read()
@@ -1049,6 +1052,9 @@
             self.branch.unlock()
             raise
 
+    def get_physical_lock_status(self):
+        return self._control_files.get_physical_lock_status()
+
     def _basis_inventory_name(self):
         return 'basis-inventory'
 

